/**
 * Copyright (c) 2008 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0,
 * and you may not use this file except in compliance with the Apache License Version 2.0.
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the Apache License Version 2.0 is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */
package org.sonatype.plexus.jetty.custom;

import org.codehaus.plexus.PlexusConstants;
import org.codehaus.plexus.PlexusContainer;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.util.component.LifeCycle;
import org.eclipse.jetty.util.log.Log;
import org.eclipse.jetty.util.log.Logger;
import org.eclipse.jetty.webapp.WebAppContext;

/**
 * Inject the Plexus container that was initialized in {@link PlexusContainerHolder} (usually, via jetty booter) into
 * {@link WebAppContext} instances that were constructed via a custom jetty.xml configuration file. It triggers if
 * {@link PlexusContainerHolder} isPlexusSet() method returns true.
 * 
 * @author jdcasey
 */
public class InjectExistingPlexusListener
    implements LifeCycle.Listener
{
    private static final Logger logger = Log.getLogger( InjectExistingPlexusListener.class.getName() );

    public void lifeCycleStarting( LifeCycle lifecycle )
    {
        if ( lifecycle instanceof ContextHandlerCollection && PlexusContainerHolder.isPlexusSet() )
        {
            final PlexusContainer container = PlexusContainerHolder.get();

            Handler[] childHandlers = ( (ContextHandlerCollection) lifecycle ).getChildHandlers();

            if ( childHandlers != null && childHandlers.length > 0 )
            {
                for ( Handler child : childHandlers )
                {
                    if ( child instanceof WebAppContext )
                    {
                        WebAppContext webapp = (WebAppContext) child;

                        if ( logger != null )
                        {
                            logger.info( "Injecting Plexus container for: {} (context path: {})",
                                webapp.getDisplayName(), webapp.getContextPath() );
                        }

                        webapp.setAttribute( PlexusConstants.PLEXUS_KEY, container );

                        webapp.setClassLoader( container.getContainerRealm() );
                    }
                }
            }
        }
    }

    public void lifeCycleStarted( LifeCycle event )
    {
        // NOP
    }

    public void lifeCycleStopped( LifeCycle event )
    {
        // NOP
    }

    public void lifeCycleStopping( LifeCycle event )
    {
        // NOP
    }

    public void lifeCycleFailure( LifeCycle event, Throwable cause )
    {
        // NOP
    }
}
